@@ -3,7 +3,7 @@ package FlyBy;
 use strict;
 use warnings;
 use 5.010;
-our $VERSION = '0.094';
+our $VERSION = '0.095';
 
 use Moo;
 
@@ -97,14 +97,17 @@ sub _from_index {
         $value = substr($value, 1);
     }
 
-    if (not $add_missing_key and not exists $index_sets->{$key}) {
-        $result = ($negated) ? $self->_full_set : $self->_full_set->empty_clone;    # Avoiding auto-viv on request
+    return $negated ? $self->_full_set : $self->_full_set->empty_clone
+        unless $add_missing_key or exists $index_sets->{$key};                      # Avoiding auto-viv on request
+
+    if ($add_missing_key) {
+        $result = $index_sets->{$key}{$value} //= $self->_full_set->empty_clone;    # Sets which do not (yet) exist in the index are null.
     } else {
-        $index_sets->{$key}{$value} //= $self->_full_set->empty_clone;              # Sets which do not (yet) exist in the index are null.
-        $result = $index_sets->{$key}{$value};
-        $result = $self->_full_set->difference($result) if ($negated);
+        $result = $index_sets->{$key}{$value} //  $self->_full_set->empty_clone;    # Sets which do not (yet) exist in the index are null.
     }
 
+    $result = $self->_full_set->difference($result) if ($negated);
+
     return $result;
 }
 
@@ -256,10 +259,7 @@ sub all_keys {
 sub values_for_key {
     my ($self, $key) = @_;
 
-    my $set = $self->index_sets->{$key};
-    my @values = grep { my $v = $set->{$_}; $v && %{$v->{elements}} } keys %$set;
-
-    return (sort { $a cmp $b } @values);
+    return (sort { $a cmp $b } keys %{$self->index_sets->{$key}});
 }
 
 1;
